@@ -6,7 +6,7 @@ from threading import Lock
 class User:
     __slots__ = ("name", "client_ver", "network_ver", "p2p_accept", "p2p_udp_accept",
                  "p2p_port", "start_time", "number", "sock", "host_port",
-                 "aeskey", "sock_type", "neers", "score", "warn", "lock")
+                 "aeskey", "sock_type", "neers", "score", "warn", "last_seen", "lock")
 
     def __init__(self, number, sock, host_port, aeskey, sock_type):
         self.name = None
@@ -25,20 +25,25 @@ class User:
         # user experience
         self.score = 0
         self.warn = 0
+        self.last_seen = int(time())
         self.lock = Lock()
 
     def __repr__(self):
         return "<User {} {}s {} score={} warn={}>".format(
-            self.name, int(time())-self.start_time, (self.host_port[0], self.p2p_port), self.score, self.warn)
+            self.name, int(time()) - self.start_time, (self.host_port[0], self.p2p_port), self.score, self.warn)
 
     def __del__(self):
         self.close()
 
     def close(self):
-        try: self.sock.shutdown(socket.SHUT_RDWR)
-        except: pass
-        try: self.sock.close()
-        except: pass
+        try:
+            self.sock.shutdown(socket.SHUT_RDWR)
+        except Exception as e:
+            pass
+        try:
+            self.sock.close()
+        except Exception as e:
+            pass
 
     def send(self, msg):
         with self.lock:
@@ -64,7 +69,8 @@ class User:
              'p2p_accept': self.p2p_accept,
              'p2p_udp_accept': self.p2p_udp_accept,
              'p2p_port': self.p2p_port,
-             'start_time': self.start_time}
+             'start_time': self.start_time,
+             'last_seen': self.last_seen}
         return r
 
     def deserialize(self, s):
@@ -75,6 +81,7 @@ class User:
         self.p2p_udp_accept = s.get('p2p_udp_accept', False)
         self.p2p_port = s['p2p_port']
         self.start_time = s['start_time']
+        self.last_seen = s.get('last_seen', self.last_seen)
 
     def get_host_port(self):
         # connection先
@@ -83,5 +90,6 @@ class User:
         return tuple(host_port)
 
     def update_neers(self, items):
-        # {(host,port): header, ..}
-        self.neers = items
+        # [[(host,port), header],..]
+        for host_port, header in items:
+            self.neers[tuple(host_port)] = header
